/**
 *  	Author: gyxoBka
 * 
 * 		------------------------------
 *		-*- NickName Control 0.9.3 -*-
 *		------------------------------
 * 
 *  	~~~~~~~~~~~
 *  	- CREDITS -
 *  	~~~~~~~~~~~
 * 		Radius for idea and help
 * 
 *  	~~~~~~~~~~~~~~~
 *  	- Description -
 *  	~~~~~~~~~~~~~~~
 * 
 * 		Plugin can block too frequent change nickname and punish it, 
 * 		find and remove special characters from nickname, 
 * 		find curse words or undesirable words and change nickname for new.
 * 
 *		~~~~~~~~~
 *		- CVARS -
 *		~~~~~~~~~
 * 		Command to punish for frequent change.
 *		 You can use:
 * 		%userid% %name% %ip%
 *		%reason%
 * 		%time%
 * 		bcn_punishment "amx_kick %userid% %reason%"
 * 
 * 		%reason%
 * 		bcn_reason "BadName"
 * 
 * 		%time%
 * 		bcn_time "30"
 * 
 * 		Max warnings to punish
 * 		bcn_warnings "10"  ( default: 3 )
 * 
 * 		Seconds before next change name
 * 		bcn_wait "10"	( default: 10 )
 * 
 * 		Check name for special characters [ 1 - Yes / 0 - No ]
 * 		bcn_checkname "1" ( default: 1)
 * 
 * 		Special characters list
 * 		bcn_symbols "+%#"
 * 
 * 		Check name for restricted words: advertisement, curse words or undesirable words [ 1 - Yes / 0 - No ]
 * 		bcn_blockname "1" ( default: 1)	
 * 
 * 		Check name by regular expressions [ 1 - Yes / 0 - No ]
 * 		bcn_regex "1" ( default: 1)	
 * 
 * 		Regular pattern
 *		bcn_pattern "(?:\w+\.[a-z]{2,4}\b|(?:\s*\d+\s*\.){3})"
 *		
 *		~~~~~~~~~~~
 *		- CONFIGS -
 *		~~~~~~~~~~~
 *
 *		addonds/amxmodx/configs/blocked_words.ini
 *		There you must write words which you want to block. One word for one line!
 *		Do not forget to erase the blank lines in a file!
 *
 *		addonds/amxmodx/configs/white_names.ini
 *		There you must write names which will be replaced after checking. One word for one line!
 *		Do not forget to erase the blank lines in a file!
 *
 *		addonds/amxmodx/configs/name_control.cfg
 *		There you can write cvars which will be loaded to your server ;)
 */

#include <amxmodx>
#include <fakemeta>
#include <regex>
#if AMXX_VERSION_NUM < 183
	#include <colorchat>
#endif	
	
#define MAX_CLIENTS 	32				// Max players num	
#define MAX_BLOCK		50				// Max blocked words num	
#define MAX_WHITE		50				// Max names for change
#define PUNISH_LENGTH	128				// Max length of punishment

enum _:Labels
{
	CVAR_WARNINGS = 0,
	CVAR_WAIT,
	CVAR_CHECKNAME,
	CVAR_BLOCK,
	CVAR_PUNISH,
	CVAR_REASON,
	CVAR_TIME,
	CVAR_REGEX,
	CVAR_PATTERN,
	CVAR_SYMBOLS
}	

new pCvar[Labels];
new g_iWarnings[MAX_CLIENTS+1];
new g_szBlock[MAX_BLOCK][32];
new g_szWhite[MAX_WHITE][32];
new g_iNextName;
new g_szPunishment[PUNISH_LENGTH + 1];
new g_szExceptSymbols[20]
new g_iSizeWhite, g_iSizeBlock;

new Float:g_flWaitTime[MAX_CLIENTS + 1];
new Regex:g_RegexID;

new const szReason[] = "[NickControl] Change name and try again."
new const szPunish[] = "kick %userid% %reason%";

public plugin_init() 
{
	register_plugin("NickName Control", "0.9.3", "gyxoBka");
	register_forward(FM_ClientUserInfoChanged,"fw_ClientUserInfoChanged", false);
}

public client_disconnect(id) 
{
	g_flWaitTime[id] = 0.0
	g_iWarnings[id] = 0
}	

public plugin_precache()
{
	LoadCvars();
	g_iSizeWhite = LoadNames("white_names.ini", g_szWhite, charsmax(g_szWhite));
	if(get_pcvar_num(pCvar[CVAR_BLOCK])) g_iSizeBlock = LoadNames("blocked_words.ini", g_szBlock, charsmax(g_szBlock));
}	
	
	
public client_putinserver(id)
{
	NickControl(id)
}	

public fw_ClientUserInfoChanged(id, szBuffer)
{
	if(!is_user_alive(id)) return FMRES_HANDLED;

	static szOldName[32], szNewName[32];
	get_user_name(id, szOldName, 31);
	engfunc(EngFunc_InfoKeyValue, szBuffer, "name", szNewName, 31);
	
	if(equali(szOldName, szNewName)) return FMRES_HANDLED;
	
	static piWaitTime; piWaitTime = get_pcvar_num(pCvar[CVAR_WAIT]); 
	static Float:flTime; flTime = get_gametime();

	if(g_flWaitTime[id] > flTime)
	{
		g_flWaitTime[id] = flTime + piWaitTime;
			
		engfunc(EngFunc_SetClientKeyValue, id, szBuffer, "name", szOldName);
			
		if(++g_iWarnings[id] >= get_pcvar_num(pCvar[CVAR_WARNINGS]))
		{
			PunishPlayer(id) 
			server_cmd("%s", g_szPunishment)
		}	
		client_print_color(id, 0, "^1[^4 Предупреждение^1 ]^3 Нельзя часто менять ник ^1[^4%d^1/^4%d^1]^3!",g_iWarnings[id], get_pcvar_num(pCvar[CVAR_WARNINGS]))
		return FMRES_HANDLED;
	}	
	g_flWaitTime[id] = flTime + piWaitTime;
	NickControl(id, szNewName)
	
	return FMRES_HANDLED;
}

LoadNames(const FileName[], ret[][], const len)
{
	new szConfigsDir[64];

	get_localinfo( "amxx_configsdir", szConfigsDir, charsmax(szConfigsDir) )
	formatex(szConfigsDir, charsmax(szConfigsDir), "%s/%s", szConfigsDir, FileName)

	new FileHandle = fopen(szConfigsDir, "rt");
	if(!FileHandle) return set_fail_state("|     [NickName Control] File config Not Found     |");	
	
	new szTempString[32], i;
	
	while( !feof(FileHandle) )
	{
		fgets(FileHandle, szTempString, 31)
		trim(szTempString);
		
		if(!szTempString[0] || szTempString[0] == ';' || szTempString[0] == '/' && szTempString[1] == '/') continue;
		
		copy(ret[i++], len, szTempString)	
	}
	fclose(FileHandle);
	
	return file_size(szConfigsDir, 1);
}	

PunishPlayer(id)
{
	get_pcvar_string(pCvar[CVAR_PUNISH], g_szPunishment, PUNISH_LENGTH);

	new szReason[PUNISH_LENGTH];
	get_pcvar_string(pCvar[CVAR_REASON], szReason, charsmax(szReason));

	new userid[10], authid[32], szName[32], ip[16], Time[6];
	formatex(userid, charsmax(userid), "#%i", get_user_userid(id));
	get_user_authid(id, authid, charsmax(authid));
	get_user_name(id, szName, charsmax(szName));
	get_user_ip(id, ip, charsmax(ip), 1);
	num_to_str(get_pcvar_num(pCvar[CVAR_TIME]), Time, charsmax(Time));

	replace_all(g_szPunishment, PUNISH_LENGTH, "%userid%", 	userid);
	replace_all(g_szPunishment, PUNISH_LENGTH, "%authid%",	authid);
	replace_all(g_szPunishment, PUNISH_LENGTH, "%name%", 	szName);
	replace_all(g_szPunishment, PUNISH_LENGTH, "%ip%", 		ip);
	replace_all(g_szPunishment, PUNISH_LENGTH, "%reason%", 	szReason);
	replace_all(g_szPunishment, PUNISH_LENGTH, "%time%",	Time);
}	

NickControl(const id, szName[32] = "")
{
	if( strlen(szName) == 0 ) get_user_name(id, szName, 31)
	
	if(get_pcvar_num(pCvar[CVAR_BLOCK]))
	{
		for(new i = 0; i < g_iSizeBlock; i++)
		{
			if( (containi(szName, g_szBlock[i]) != -1) )
			{
				formatex(szName, 31, "%s", g_szWhite[g_iNextName++])
				if( g_iNextName >= g_iSizeWhite ) g_iNextName = 0	
				break;
			}
		}	
	}
	if( get_pcvar_num(pCvar[CVAR_REGEX]))
	{
		new ret;
		if( regex_match_c( szName, g_RegexID, ret) > 0)
		{
			formatex(szName, 31, "%s", g_szWhite[g_iNextName++])
			if( g_iNextName >= g_iSizeWhite ) g_iNextName = 0
		}	
	}	
	
	if(get_pcvar_num(pCvar[CVAR_CHECKNAME]))
	{
		new NewName[32];
		new i, a, t;
		new b = strlen(g_szExceptSymbols)
		new bool:bSkip = false
		while( ( t = szName[i++]) )
		{
			bSkip = false
			for(new j; j < b; j++)
			{
				if( t == g_szExceptSymbols[j] )
				{
					bSkip = true
					break;
				}
			}
			if(!bSkip) NewName[a++] = t	
		}	
		formatex(szName, 31, "%s", NewName)	
	}
	engfunc(EngFunc_SetClientKeyValue, id, engfunc(EngFunc_GetInfoKeyBuffer, id), "name", szName)
}

LoadCvars()
{	
	pCvar[CVAR_WARNINGS] 	= register_cvar( "bcn_warnings", 	"3"		);
	pCvar[CVAR_WAIT] 		= register_cvar( "bcn_wait", 		"10"	);
	pCvar[CVAR_CHECKNAME] 	= register_cvar( "bcn_checkname", 	"1"		);
	pCvar[CVAR_BLOCK]		= register_cvar( "bcn_blockname", 	"1"		);
	pCvar[CVAR_REGEX]		= register_cvar( "bcn_regex", 		"1"		);
	pCvar[CVAR_PUNISH] 		= register_cvar( "bcn_punishment", 	szPunish);
	pCvar[CVAR_REASON] 		= register_cvar( "bcn_reason", 		szReason);
	pCvar[CVAR_TIME] 		= register_cvar( "bcn_time", 		"30"	);
	pCvar[CVAR_PATTERN] 	= register_cvar( "bcn_pattern", 	"123"	);	
	pCvar[CVAR_SYMBOLS]		= register_cvar( "bcn_symbols", 	"123"	);	
	
	
	new szConfigsDir[64];
	get_localinfo( "amxx_configsdir", szConfigsDir, charsmax(szConfigsDir) )
	server_cmd("exec %s/name_control.cfg", szConfigsDir);
	server_exec();
	
	if( get_pcvar_num(pCvar[CVAR_REGEX]) )
	{
		new szPattern[128];
		new ret, error[128]
		get_pcvar_string(pCvar[CVAR_PATTERN], szPattern, charsmax(szPattern))
		g_RegexID = regex_compile( szPattern, ret, error, charsmax(error), "i" )
		
		if (g_RegexID == REGEX_PATTERN_FAIL) return set_fail_state("|     Incorrect pattern.    |");	
	}
	get_pcvar_string(pCvar[CVAR_SYMBOLS], g_szExceptSymbols, charsmax(g_szExceptSymbols) )
	return 0;
}	